(*
 *  beware IRC services, authaccount.pas
 *  Copyright (C) 2002 Bas Steendijk
 *
 *  modifications by peter green
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
unit authaccount;

{
possible load accounts code:

[A.account]
- currentaccount := nil

name=
- if accounts exists, make it current, otherwise, create new account, add it to list, and make it current

password=
- crypted password

level=
- change level of current account

mask=mask1 mask2 mask3
- set mask list of account to these masks, remove existing masks
maskmore=mask1 mask2 mask3
- add masks to list, check for already existing masks


- an account without password is disabled (cant login)
set with MODOPER SETPASS idnick <newpass>

if the account is mentioned in the cfg file, set a flag. at the end of loading,
delete all accounts which dont have the flag set 
}


interface

uses blinklist,bstuff,buser;

const
  idnicklen=16;

type
  tAaccount=class(tlinklist)
    name:string;
    password:string;
    requestedpass:string; {extra, temp, password}
    level:integer;
    mask:tstringlinklist;

    user:tplinklist;
    {who is logged in to this account, linked list of pointers, nil=none}

    marked:boolean; {internal}
    operonly:boolean;
    email : string; {email set on registration can only be changed by admins}
    hellouser : string; {  "nick!user@host ip" form
                         set on registration with hello command
                         empty string for accounts created by ircops
                         cannot be changed by anyone. used to issue glines
                         when a user creates an account useing someone elses
                         email (this is much safer than trusting the
                         nick!user@host in the bounce/reply}

    lastseents:integer; {for expire purposes}
    lastmailts:integer; {requestpass throttle}

    function addmask(s:string):boolean;
    function delmask(s:string):boolean;
    function adduser(us:tuser):boolean;
    function removeuser(us:tuser):boolean;
    procedure clearmasks;
    constructor create;
    destructor destroy; override;
  end;

function findAaccount(const s:string):tAaccount;

{add account. only to be used from command handler or similar}
function addAaccount(const name:string;level:integer):tAaccount;

function validaccountname(const s:string):boolean;
function saneaccountname(const s:string):boolean;

function convertaccountname(const s:string):string;

procedure clearAaccount;
procedure saveAaccount(const filename:string);
procedure loadAaccount(const filename:string);

procedure updatevalidaccounttable;

function authaccountpasswordmatch(p:taaccount;password:string):boolean;

var
  Aaccountlist:tAaccount;
  validaccounttable:array[0..1,0..255] of boolean;

implementation

uses authbot,bconfig,passcryp,bparse,bcmds,sysutils,unitwritetxt;

function validaccountname(const s:string):boolean;
var
  a:integer;
begin
  result := false;
  if (s = '') or (length(s) > locopt.accountlen) then exit;
  if not validaccounttable[0,ord(s[1])] then exit;
  for a := 2 to length(s) do begin
    if not validaccounttable[1,ord(s[a])] then exit;
  end;
  result := true;
end;

function saneaccountname(const s:string):boolean;
var
  a:integer;
begin
  result := false;
  if (s = '') or (length(s) > 63) then exit;
  for a := 1 to length(s) do if s[a] <= #32 then exit;
  result := true;
end;

procedure updatevalidaccounttable;

procedure filltable(num:integer;const s:string);
var
  a,b:integer;
begin
  fillchar(validaccounttable[num],sizeof(validaccounttable[num]),0);
  for a := 1 to length(s) do begin
    case s[a] of
      'A':for b := ord('A') to ord('Z') do validaccounttable[num,b] := true;
      'a':for b := ord('a') to ord('z') do validaccounttable[num,b] := true;
      '0':for b := ord('0') to ord('9') do validaccounttable[num,b] := true;
      else validaccounttable[num,ord(s[a])] := true;
    end;
  end;
end;

var
  a:integer;
  s:string;
begin
  s := locopt.accountchars;
  a := pos(' ',s);
  assert(a <> 0,'updatevalidaccounttable: no space in accountchars string');
  filltable(0,copy(s,1,a-1));
  filltable(1,copy(s,a+1,length(s)));
end;

{
centralized code to convert (or not) to uppercase
}
function convertaccountname(const s:string):string;
begin
  result := ircupper(s);
end;
{
IDnick's are case tolerant here
i may later make hashtable code for this,
but there arent thousands of accounts anyway, so this will do for now
}
function findAaccount(const s:string):tAaccount;
var
  p:tAaccount;
begin
  p := Aaccountlist;
  result := nil;
  while p <> nil do begin
    if strcompup(p.name,s) then begin
      result := p;
      exit
    end;
    p := tAaccount(p.next);
  end;
end;


function authaccountpasswordmatch(p:taaccount;password:string):boolean;
begin
  result := ((p.password <> '') and passmatch(password,p.password))
  or ((p.requestedpass <> '') and passmatch(password,p.requestedpass))
end;


function addAaccount(const name:string;level:integer):tAaccount;
var
  p:tAaccount;
begin
  result := nil;
  if findAaccount(name) <> nil then exit; {aready exists}
  p := tAaccount.create;
  linklistadd(tlinklist(Aaccountlist),tlinklist(p));
  p.name := name;
  p.level := level;
  result := p;
end;

function tAaccount.addmask(s:string):boolean;
var
  s2:string;
  sl:tstringlinklist;

begin
  result := false;

  s := cookmask(s);
  {process mask (like ban), must also check here if mask is valid}

  s2 := ircupper(s);
  sl := self.mask;
  while sl <> nil do begin
    if ircupper(sl.s) = s2 then exit; {mask already exists}
    sl := tstringlinklist(sl.next);
  end;

  sl := tstringlinklist.create;
  sl.s := s;
  linklistadd(tlinklist(self.mask),tlinklist(sl));
  result := true;
end;

function tAaccount.delmask(s:string):boolean;
var
  s2:string;
  sl,sl2:tstringlinklist;

begin
  result := false;

  s := cookmask(s);
  {process mask (like ban), must also check here if mask is valid}

  s2 := ircupper(s);
  sl2 := nil;
  sl := self.mask;
  while sl <> nil do begin
    if ircupper(sl.s) = s2 then begin
      sl2 := sl;
      break;
    end;
    sl := tstringlinklist(sl.next);
  end;

  if sl2 = nil then exit; {mask doesn't exist}
  linklistdel(tlinklist(self.mask),tlinklist(sl2));
  sl2.destroy;
  result := true;
end;

constructor tAaccount.create;
begin
  inherited create;
  {}
end;

destructor tAaccount.destroy;
begin
  while self.user <> nil do authbot.deauth(tuser(self.user.p));

  {if self.user <> nil then authbot.deauth(self.user);}

  self.clearmasks;
  inherited destroy;
end;

function TAaccount.adduser(us:tuser):boolean;
var
  p:tplinklist;
begin
  result := false;
  {search user already in list}
  p := self.user;
  while p <> nil do begin
    if p.p = us then exit;
    p := tplinklist(p.next);
  end;
  result := true;
  p := tplinklist.create;
  p.p := us;
  linklistadd(tlinklist(self.user),tlinklist(p));
end;

function TAaccount.removeuser(us:tuser):boolean;
var
  p:tplinklist;
begin
  result := true;
  {search user in list}
  p := tplinklist(self.user);
  while p <> nil do begin
    if p.p = us then begin
      linklistdel(tlinklist(self.user),tlinklist(p));
      p.destroy;
      exit;
    end;
    p := tplinklist(p.next);
  end;
  result := false;
end;

procedure tAaccount.clearmasks;
var
  sl,sl2:tstringlinklist;
begin
  {destroy all masks}
  sl := self.mask;
  while sl <> nil do begin
    sl2 := tstringlinklist(sl.next);
    linklistdel(tlinklist(self.mask),tlinklist(sl));
    sl.destroy;
    sl := sl2;
  end;
end;

{free A oper account list (to be done on shutdown)}
procedure clearAaccount;
var
  p,p2:tAaccount;
begin
  p := Aaccountlist;
  while p <> nil do begin
    p2 := tAaccount(p.next);
    linklistdel(tlinklist(Aaccountlist),tlinklist(p));
    p.destroy;
    p := p2;
  end;
end;

var
  currentaccount:tAaccount;

procedure loadAaccountproc(sv,sn:string);
begin
  if sv = 'NAME' then begin
    currentaccount := nil;
    if not saneaccountname(convertaccountname(sn)) then exit;

    currentaccount := findAaccount(sn);
    if currentaccount = nil then begin
      currentaccount := tAaccount.create;
      linklistadd(tlinklist(Aaccountlist),tlinklist(currentaccount));
    end else begin
      currentaccount.clearmasks;
    end;
    currentaccount.name := sn; {if the account already exists, this allows changing the case of the name}
    currentaccount.marked := true;
  end else if currentaccount <> nil then begin
    if sv = 'LEVEL' then begin
      currentaccount.level := strtointdef(sn,0);
    end else if sv = 'MASK' then begin
      sn := cookmask(sn);
      if validmask(sn) then currentaccount.addmask(sn);
    end else if sv = 'PASSWORD' then begin
      if iscryptedlength(length(sn)) or (sn = '') then
      currentaccount.password := sn
      else
      currentaccount.password := passnewcrypt(sn);
    end else if sv = 'OPERONLY' then begin
      currentaccount.operonly := sn = '1';
    end else if sv = 'EMAIL' then begin
      currentaccount.email := sn;
    end else if sv = 'HELLOUSER' then begin
      currentaccount.hellouser := sn;
    end else if sv = 'LASTSEEN' then begin
      currentaccount.lastseents := strtointdef(sn,0);
    end else if sv = 'LASTMAIL' then begin
      currentaccount.lastmailts := strtointdef(sn,0);
    end;

  end;

end;


procedure loadAaccount(const filename:string);
var
  p,p2:tAaccount;

begin
  //writeln('loadAaccount called');
  {don't clear list}

  {un-mark all accounts}
  p := Aaccountlist;
  while p <> nil do begin
    p.marked := false;
    p := tAaccount(p.next);
  end;

  currentaccount := nil;
  readcfg(filename,@loadAaccountproc);
  currentaccount := nil;

  p := Aaccountlist;
  while p <> nil do begin
    p2 := tAaccount(p.next);
    if not p.marked then begin
      linklistdel(tlinklist(Aaccountlist),tlinklist(p));
      p.destroy;
    end;
    p := p2;
  end;
end;

procedure saveAaccount(const filename:string);
var
  t:textfile;
  sl:tstringlinklist;
  p:tAaccount;
begin
  if not openforwrite(t,filename) then exit;
  p := tAaccount(Aaccountlist);
  while p <> nil do begin
    {}
    writeln(t,'[account]');
    writeln(t,'name='+p.name);
    writeln(t,'level='+inttostr(p.level));
    writeln(t,'password='+p.password);
    writeln(t,'operonly='+inttostr(ord(p.operonly)));
    writeln(t,'email='+p.email);
    writeln(t,'hellouser='+p.hellouser);
    writeln(t,'lastseen='+inttostr(p.lastseents));
    writeln(t,'lastmail='+inttostr(p.lastmailts));
    sl := p.mask;
    while sl <> nil do begin
      writeln(t,'mask='+sl.s);
      sl := tstringlinklist(sl.next);
    end;
    p := tAaccount(p.next);
    writeln(t);
  end;
  closefile(t);
end;

end.
