(*
 *  beware IRC services, invkickbot.pas
 *  Copyright (C) 2002 Bas Steendijk
 *
 *  modifications by peter green
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
unit invkickbot;

interface
uses bservicemngr,pgtypes,buser,bparse,bstuff,bcmds,bchannel,bservunit,blinklist;
type
  tinvkickbot=class               // note : this class is designed never to be
                                  // destroyed
    srvunit  : tserviceunit;
    up       : Boolean;
    mybot    : tuser;
    nick     : string;
    userid   : string;
    realname : string;
    host     : string;
    prefix   : string;

    constructor create;
    procedure setupsrvunit;virtual;
    // note : userleavechannel does not fire for the bot itself leaving the channel
    // that is what botleavechannel is for
    procedure userleavechannel(ch:tchannel;user:tuser);virtual;
    procedure botleavechannel(ch:tchannel);virtual;
  end;

implementation
procedure onstartup(tag:taddrint);
var
  currentuser : tuser;
begin
  with tinvkickbot(tag) do begin
    if up then exit;
    up := true;
    mybot := addbot(srvunit,nick,userid,host,realname,0);
    prefix := sendprefixstr(mybot);
  end;
end;
procedure onshutdown(tag:taddrint);
var
  currentuser : tuser;
begin
  with tinvkickbot(tag) do begin
    if not up then exit;
    up := false;

    bot_quit(mybot,shutdownreason);
    mybot := nil;
  end;
end;

procedure ondestroyuser(us:tuser;tag:taddrint);
var
  uc : tlinklist;
begin
  with tinvkickbot(tag) do begin
    uc := us.channel;
    while uc <> nil do begin
      if (tuserchan(uc).ch.usercount =2) and (uc<>mybot) then begin
        sendparse(mybot.idstr+' '+tok_part+' '+tuserchan(uc).ch.name);
      end;
      if us=mybot then begin
        botleavechannel(tuserchan(uc).ch);
      end else begin
        userleavechannel(tuserchan(uc).ch ,us);
      end;
      uc := uc.next;
    end
  end;
end;

procedure oninvite(sptr:tuser;user:tuser;channel:tchannel;tag:taddrint);
begin
  with tinvkickbot(tag) do begin
    if not isonchannel(mybot,channel) then begin
      sendparse(mybot.idstr+' '+tok_join+' '+channel.name);
    end;
  end;
end;

procedure onleavechannel(channel:tchannel;source:tuser;target:tuser;kick:boolean;message:string;tag:taddrint);


begin
  with tinvkickbot(tag) do begin
    if (channel.usercount =2) and (target<>mybot) then begin
      // 2 users = me and the user who is leaveing

      sendparse(mybot.idstr+' '+tok_part+' '+channel.name );
    end;
    if target=mybot then begin
      botleavechannel(channel);
    end else begin
      userleavechannel(channel,target);
    end;
  end;
end;


constructor tinvkickbot.create;
begin
  inherited create;
  srvunit := createserviceunit;
  setupsrvunit;
  if not registerserviceunit(srvunit) then raise exception.create('invkickbot registerserviceunit failed');
  nick :=  'invkickbot';
  userid := 'invkickbot';
  realname := 'invkickbot';
end;
procedure tinvkickbot.setupsrvunit;
begin
  srvunit.name := 'invkickbot';
  srvunit.onstartup := onstartup;
  //srvunit.onparse   := onparse;
  srvunit.oninvite  := oninvite;
  srvunit.onleavechannel := onleavechannel;
  srvunit.onshutdown := onshutdown;
  //srvunit.onprivmsg := onprivmsg;
  //srvunit.ontimer := ontimer;
  //srvunit.onsave := onsave;
  //srvunit.onaccount := onaccount;
  srvunit.ondestroyuser := ondestroyuser;
  //srvunit.onrehash := onrehash;
  srvunit.tag := taddrint(self);

end;



procedure tinvkickbot.userleavechannel(ch:tchannel;user:tuser);
begin
  //stub
end;

procedure tinvkickbot.botleavechannel(ch:tchannel);
begin
  //stub
end;

initialization
  {tinvkickbot.create;} // this line allows invkickbot to be loaded itself for
                        // test perposes
end.
